package ext.util {
import ext.Base;

[Native("Ext.util.Filter", require)]
/**
 * Represents a filter that can be applied to a →<code>ext.util.MixedCollection</code>. Can either simply
 * filter on a property/value pair or pass in a filter function with custom logic. Filters are always used in the
 * context of MixedCollections, though →<code>ext.data.Store</code>s frequently create them when filtering and searching
 * on their records. Example usage:
 * <pre>
 * // Set up a fictional MixedCollection containing a few people to filter on
 * var allNames = new Ext.util.MixedCollection();
 * allNames.addAll([
 *     { id: 1, name: 'Peter',  age: 25 },
 *     { id: 2, name: 'Egon',   age: 37 },
 *     { id: 3, name: 'Ray',    age: 32 },
 *     { id: 4, name: 'Winston',age: 26 }
 * ]);
 *
 * var ageFilter = new Ext.util.Filter({
 *     property: 'age',
 *     value   : 32
 * });
 *
 * var longNameFilter = new Ext.util.Filter({
 *     filterFn: function(item) {
 *         return item.name.length &gt; 4;
 *     }
 * });
 *
 * // a new MixedCollection with the 2 names longer than 4 characters
 * var longNames = allNames.filter(longNameFilter);
 *
 * // a new MixedCollection with the 1 person of age 32:
 * var youngFolk = allNames.filter(ageFilter);
 * </pre>
 * @see ext.util.MixedCollection
 * @see ext.data.Store
 * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html Original Ext JS documentation of 'Ext.util.Filter'
 */
public class Filter extends ext.Base {
  /**
   * Creates a single filter function which encapsulates the passed Filter array or
   * Collection.
   * @param filters The filters from which to
   * create a filter function.
   * @return A function, which when passed a candidate object returns <code>true</code>
   * if the candidate passes all the specified Filters.
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#static-method-createFilterFn Original Ext JS documentation of 'createFilterFn'
   */
  public static native function createFilterFn(filters:*):Function;

  /**
   * Checks if two filters have the same properties (Property, Operator and Value).
   * @since 6.2.0
   * @param filter1 The first filter to be compared
   * @param filter2 The second filter to be compared
   * @return <code>true</code> if they have the same properties.
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#static-method-isEqual Original Ext JS documentation of 'isEqual'
   */
  public static native function isEqual(filter1:ext.util.Filter, filter2:ext.util.Filter):Boolean;

  /**
   * Creates new Filter.
   * @param config Config object
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#method-constructor Original Ext JS documentation of 'constructor'
   */
  public function Filter(config:ext.util.Filter = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to allow any match - no regex start/end line anchors will be added.
   * @default false
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-anyMatch Original Ext JS documentation of 'anyMatch'
   * @see #getAnyMatch()
   * @see #setAnyMatch()
   */
  public native function get anyMatch():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set anyMatch(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to make the regex case sensitive (adds 'i' switch to regex).
   * @default false
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-caseSensitive Original Ext JS documentation of 'caseSensitive'
   * @see #getCaseSensitive()
   * @see #setCaseSensitive()
   */
  public native function get caseSensitive():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set caseSensitive(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to not have this filter participate in the filtering process when the →<code>value</code> of
   * this the filter is empty according to →<code>ext.Ext.isEmpty()</code>.
   * @default false
   * @since 5.1.0
   * @see #value
   * @see ext.SExt#isEmpty() ext.Ext.isEmpty()
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-disableOnEmpty Original Ext JS documentation of 'disableOnEmpty'
   * @see #getDisableOnEmpty()
   * @see #setDisableOnEmpty()
   */
  public native function get disableOnEmpty():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disableOnEmpty(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Setting this property to <code>true</code> disables this individual Filter so that it no longer contributes to a <i>Store's filter set</i> (→<code>ext.data.Store.filters</code>)
   * <p>When disabled, the next time the store is filtered, the Filter plays no part in filtering and records eliminated by it may rejoin the dataset.</p>
   * @default false
   * @see ext.data.Store#filters
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-disabled Original Ext JS documentation of 'disabled'
   * @see #getDisabled()
   * @see #setDisabled()
   */
  public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabled(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to force exact match (^ and $ characters added to the regex). Ignored if anyMatch is true.
   * @default false
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-exactMatch Original Ext JS documentation of 'exactMatch'
   * @see #getExactMatch()
   * @see #setExactMatch()
   */
  public native function get exactMatch():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set exactMatch(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A custom filter function which is passed each item in the →<code>ext.util.MixedCollection</code> in turn. Should return
   * <code>true</code> to accept each item or <code>false</code> to reject it.
   * @see ext.util.MixedCollection
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-filterFn Original Ext JS documentation of 'filterFn'
   * @see #getFilterFn()
   * @see #setFilterFn()
   */
  public native function get filterFn():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set filterFn(value:Function):void;

  /**
   * Mutation counter which is incremented when the filter changes in ways that mean reevaluation of
   * the filtered state is necessary.
   * @default 0
   * @since 6.5.0
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#property-generation Original Ext JS documentation of 'generation'
   */
  public native function get generation():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An identifier by which this Filter is indexed in a <i>Store's filters collection</i> (→<code>ext.data.Store.filters</code>)
   * <p>Identified Filters may be individually removed from a Store's filter set by using →<code>ext.data.Store.removeFilter()</code>.</p>
   * <p>Anonymous Filters may be removed en masse by passing <code>null</code> to →<code>ext.data.Store.removeFilter()</code>.</p>
   * @see ext.data.Store#filters
   * @see ext.data.Store#removeFilter()
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-id Original Ext JS documentation of 'id'
   * @see #getId()
   * @see #setId()
   */
  public native function get id():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set id(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The operator to use to compare the →<code>property</code> to this Filter's →<code>value</code>.
   * <p>Possible values are:</p>
   * <ul>
   * <li><code>&lt;</code></li>
   * <li><code>&lt;=</code></li>
   * <li><code>=</code></li>
   * <li><code>&gt;=</code></li>
   * <li><code>&gt;</code></li>
   * <li><code>!=</code></li>
   * <li><code>in</code></li>
   * <li><code>notin</code></li>
   * <li><code>like</code></li>
   * <li>/=</li>
   * </ul>
   * <p>The <code>in</code> and <code>notin</code> operator expects this filter's →<code>value</code> to be an array and matches
   * values that are present in that array.</p>
   * <p>The <code>like</code> operator matches values that contain this filter's →<code>value</code> as a
   * substring.</p>
   * <p>The <code>'&#42;='</code> operator uses the →<code>value</code> as the source for a <code>→RegExp</code> and tests whether the
   * candidate value matches the regular expression.</p>
   * @see #property
   * @see #value
   * @see RegExp
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-operator Original Ext JS documentation of 'operator'
   * @see #getOperator()
   * @see #setOperator()
   */
  public native function get operator():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set operator(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The property to filter on. Required unless a →<code>filterFn</code> is passed.
   * @see #filterFn
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-property Original Ext JS documentation of 'property'
   * @see #getProperty()
   * @see #setProperty()
   */
  public native function get property():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set property(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional root property. This is mostly useful when filtering a Store, in which case we set the root to 'data' to
   * make the filter pull the →<code>property</code> out of the data object of each item
   * @see #property
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-root Original Ext JS documentation of 'root'
   * @see #getRoot()
   * @see #setRoot()
   */
  public native function get root():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set root(value:String):void;

  [ExtConfig]
  /**
   * The context (<code>this</code> property) in which the filtering function is called. Defaults
   * to this Filter object.
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-scope Original Ext JS documentation of 'scope'
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A function to post-process any serialization. Accepts a filter state object
   * containing <code>→property</code>, <code>→value</code> and <code>→operator</code> properties, and may either
   * mutate it, or return a completely new representation. Returning a falsey
   * value does not modify the representation.
   * @since 6.2.0
   * @see #property
   * @see #value
   * @see #operator
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-serializer Original Ext JS documentation of 'serializer'
   * @see #getSerializer()
   * @see #setSerializer()
   */
  public native function get serializer():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set serializer(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The value you want to match against. Required unless a →<code>filterFn</code> is passed.
   * <p>Can be a regular expression which will be used as a matcher or any other value
   * such as an object or an array of objects. This value is compared using the configured
   * →<code>operator</code>.</p>
   * @default null
   * @see #filterFn
   * @see #operator
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#cfg-value Original Ext JS documentation of 'value'
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:*):void;

  /**
   * Returns the value of <code>anyMatch</code>.
   * @see #anyMatch
   */
  public native function getAnyMatch():Boolean;

  /**
   * Returns the value of <code>caseSensitive</code>.
   * @see #caseSensitive
   */
  public native function getCaseSensitive():Boolean;

  /**
   * Returns the value of <code>disableOnEmpty</code>.
   * @see #disableOnEmpty
   */
  public native function getDisableOnEmpty():Boolean;

  /**
   * Returns the value of <code>disabled</code>.
   * @see #disabled
   */
  public native function getDisabled():Boolean;

  /**
   * Returns the value of <code>exactMatch</code>.
   * @see #exactMatch
   */
  public native function getExactMatch():Boolean;

  /**
   * Returns the value of <code>filterFn</code>.
   * @see #filterFn
   */
  public native function getFilterFn():Function;

  /**
   * Returns the value of <code>id</code>.
   * @see #id
   */
  public native function getId():String;

  /**
   * Returns the value of <code>operator</code>.
   * @see #operator
   */
  public native function getOperator():String;

  /**
   * Returns the value of <code>property</code>.
   * @see #property
   */
  public native function getProperty():String;

  /**
   * Returns the value of <code>root</code>.
   * @see #root
   */
  public native function getRoot():String;

  /**
   * Returns the value of <code>serializer</code>.
   * @see #serializer
   */
  public native function getSerializer():Function;

  /**
   * Returns this filter's state.
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#method-getState Original Ext JS documentation of 'getState'
   */
  public native function getState():Object;

  /**
   * Returns the value of <code>value</code>.
   * @see #value
   */
  public native function getValue():*;

  /**
   * Returns this filter's serialized state. This is used when transmitting this filter
   * to a server.
   * @see https://docs.sencha.com/extjs/6.5.3/classic/Ext.util.Filter.html#method-serialize Original Ext JS documentation of 'serialize'
   */
  public native function serialize():Object;

  /**
   * Sets the value of <code>anyMatch</code>.
   * @param anyMatch The new value.
   * @see #anyMatch
   */
  public native function setAnyMatch(anyMatch:Boolean):void;

  /**
   * Sets the value of <code>caseSensitive</code>.
   * @param caseSensitive The new value.
   * @see #caseSensitive
   */
  public native function setCaseSensitive(caseSensitive:Boolean):void;

  /**
   * Sets the value of <code>disableOnEmpty</code>.
   * @param disableOnEmpty The new value.
   * @see #disableOnEmpty
   */
  public native function setDisableOnEmpty(disableOnEmpty:Boolean):void;

  /**
   * Sets the value of <code>disabled</code>.
   * @param disabled The new value.
   * @see #disabled
   */
  public native function setDisabled(disabled:Boolean):void;

  /**
   * Sets the value of <code>exactMatch</code>.
   * @param exactMatch The new value.
   * @see #exactMatch
   */
  public native function setExactMatch(exactMatch:Boolean):void;

  /**
   * Sets the value of <code>filterFn</code>.
   * @param filterFn The new value.
   * @see #filterFn
   */
  public native function setFilterFn(filterFn:Function):void;

  /**
   * Sets the value of <code>id</code>.
   * @param id The new value.
   * @see #id
   */
  public native function setId(id:String):void;

  /**
   * Sets the value of <code>operator</code>.
   * @param operator The new value.
   * @see #operator
   */
  public native function setOperator(operator:String):void;

  /**
   * Sets the value of <code>property</code>.
   * @param property The new value.
   * @see #property
   */
  public native function setProperty(property:String):void;

  /**
   * Sets the value of <code>root</code>.
   * @param root The new value.
   * @see #root
   */
  public native function setRoot(root:String):void;

  /**
   * Sets the value of <code>serializer</code>.
   * @param serializer The new value.
   * @see #serializer
   */
  public native function setSerializer(serializer:Function):void;

  /**
   * Sets the value of <code>value</code>.
   * @param value The new value.
   * @see #value
   */
  public native function setValue(value:*):void;
}
}